#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
 * ------------------------------------------------------------------------
 * bmTraces
 * ------------------------------------------------------------------------
 * @license : GNU Affero General Public License Version 3 <http://www.gnu.org/licenses>
 * @author  : Pascal SALAUN 
 * @Website : http://www.bm-stats.org <http://www.bm-stats.org>
 * ------------------------------------------------------------------------
"""

# Native Python Lib/Class 
import cStringIO
from collections import defaultdict
import magic
import os
import re
import tarfile
import subprocess
import sys


# bmTraces Class
from fileMIMEType import *
from sqlQuery import *


io_method = cStringIO.StringIO


class mailLogParser:
        
        def __init__(self, userOwnDb, searchFile, searchCase, searchStr):
                self.user          = userOwnDb
                self.searchFile    = searchFile.encode('utf8')
                self.searchCase    = searchCase
                self.searchStr     = searchStr
                self.cDb            = sqlQuery(self.user)
                self.cDb.initUserDb()
                self.searchPattern = re.compile(r".*" + str(self.searchStr) + ".*")

                """
                #Postfix : these lines are about postfix activity for one mail  
                """        
                ## Get smtpd queueid on connection
                #Aug 22 15:38:15 galadriel postfix/smtpd[29211]: 7E83C9E4318: client=.*
                self.smtpdLine   = re.compile(r"(?P<date>.{6}) (?P<hour>.{8}) (.*)(postfix/smtpd)\[(\d+)\]: (?P<queueid>.*): client=(?P<server>.*)(?:(, .*)+)?")

                ## Get message-id on receiving message
                #Aug 22 20:09:21 galadriel postfix/cleanup[11415]: 6C54B9E433D: message-id=<20090822200921.9q4fg64u0wg4swkw@mail.mclabosse.fr>
                self.cleanupLine = re.compile(r"(?P<date>.{6}) (?P<hour>.{8}) (.*)(postfix/cleanup)\[(\d+)\]: (?P<queueid>.*): message-id=(?P<messageid>.*)")

                ## Get filename on trace line (cf postfix "main.cf" file) 
                # Sep  5 10:16:10 galadriel postfix/cleanup[1302]: 6B1E32A87CE: warning: header Content-Type: image/png;? name="pascal.salaun_nbAtt_.png" from unknown[192.168.1.4]; \
                #        from=<frodon.saquet@laposte.net> to=<frodon.saquet@laposte.net> proto=ESMTP helo=<[192.168.1.4]>: ThisFileName=pascal.salaun_nbAtt_.png
                self.filenameLine = re.compile(r"(?P<date>.{6}) (?P<hour>.{8}) (.*)(postfix/cleanup)\[(\d+)\]: (?P<queueid>.*): warning.*: ThisFileName=(?P<filename>.*)")

                ## Get originator, volume and number of recip 
                # Aug 22 15:38:15 galadriel postfix/qmgr[4158]: 7E83C9E4318: from=<frodon.saquet@laposte.net>, size=1767, nrcpt=1 (queue active)
                self.qmgrLine = re.compile(r"(?P<date>.{6}) (?P<hour>.{8}) (.*)(postfix/qmgr)\[(\d+)\]: (?P<queueid>.*): from=<(?P<from>.*)>, size=(?P<size>\d+), nrcpt=(?P<nrcpt>\d+).*")

                ## Get list of destinators (dsn=2.*.*)
                # Apr  9 09:40:51 uruviel postfix/smtp[13879]: 22272341A57: to=<frodon.saquet@laposte.net>, relay=127.0.0.1[127.0.0.1]:10024, delay=7.6, delays=2.1/0.03/0.04/5.5, dsn=2.0.0, status=sent (250 2.0.0 from MTA(smtp:[127.0.0.1]:10025): 250 2.0.0 Ok: queued as 2004B341A80)
                self.smtpLineDSN2 = re.compile(r"(?P<date>.{6}) (?P<hour>.{8}) (.*)(postfix/)(.*)\[(\d+)\]: (?P<queueid>.*): to=<(?P<recip>.*?)(?:>, orig_to=<(?P<orig>.*?))?>, (.*)status=sent.*")

                ## Get list of deferred destinator (dsn=4.*.*)
                self.smtpLineDSN4 = re.compile(r"(?P<date>.{6}) (?P<hour>.{8}) (.*)(postfix/)(.*)\[(\d+)\]: (?P<queueid>.*): to=<(?P<recip>.*?)(?:>, orig_to=<(?P<orig>.*?))?>, (relay=.*)?(.*)status=deferred.*")

                ## Get list of deferred destinator (dsn=5.*.*)
                self.smtpLineDSN5 = re.compile(r"(?P<date>.{6}) (?P<hour>.{8}) (.*)(postfix/)(.*)\[(\d+)\]: (?P<queueid>.*): to=<(?P<recip>.*?)(?:>, orig_to=<(?P<orig>.*?))?>, (relay=.*)?(.*)status=bounced.*")        


                ## Get smtpd queueid on connection
                # Apr 30 00:07:56 uruviel postfix/qmgr[4321]: 5141E341A1B: removed
                self.smtpRemoved = re.compile(r"(?P<date>.{6}) (?P<hour>.{8}) (.*)(postfix/qmgr)\[(\d+)\]: (?P<queueid>.*): removed")

                """
                Cyrus/IMAP : this line is relative to a délivery success
                """
                # Apr 30 02:49:38 uruviel cyrus/lmtp[20806]: Delivered: <20160409004835.40466.indimail@im19.nocalive.com> to mailbox: mclabosse.fr!user.pascal^salaun
                self.cyrusDelivered = re.compile(r"(?P<date>.{6}) (?P<hour>.{8}) (.*)(cyrus/lmtp)\[(\d+)\]: Delivered: (?P<messageid>.*) to mailbox: (?P<domain>.*)\!user.(?P<user>.*)")
                



        def getFileContent(self):
                self.compressionCase = {"text/plain": "plaintext","application/x-gzip" : "r:gz", "application/gzip" : "r:gz", "application/x-bzip2" : "r:bz2" }
                t = fileMIMEType()
                self.ftype =ftype = t.getType(self.searchFile)
                
                if self.compressionCase[self.ftype] == "plaintext":
                        f = open(self.searchFile , 'r' )
                        self.content = f.readlines()        
                        f.close()        
                else:
                        try:
                                ft = tarfile.open(self.searchFile, self.compressionCase[ftype])
                                member0 = ft.getnames()[0]
                                f=ft.extractfile(member0)
                                self.content = f.readlines()
                                ft.close()
                                
                        except tarfile.ReadError:
                                p = subprocess.Popen(["zcat", self.searchFile], stdout = subprocess.PIPE)
                                self.content = io_method(p.communicate()[0])
                                assert p.returncode == 0
                        
                self.content
        
        def parseContent(self):
                
                data            = defaultdict(dict)
                surveyQueuid    = []
                surveyMessageid = defaultdict(dict)
        
                for line in self.content:
                        thisLine = line.rstrip()
                        
                        smtpdLineEntries      = self.smtpdLine.match(thisLine) 
                        if smtpdLineEntries is not None:
                                
                                queueid = smtpdLineEntries.group('queueid')
                                server  = smtpdLineEntries.group('server').split(",")[0].replace("["," : ").replace("]","")
                                
                                data[queueid]['server'] = server
                                data[queueid]['from'] = data[queueid]['nrcpt'] = data[queueid]['size'] = ""
                                                                                                
                                next
                                
                        cleanupLineEntries    = self.cleanupLine.match(thisLine)        
                        if cleanupLineEntries is not None:
                                date      = cleanupLineEntries.group('date')
                                hour      = cleanupLineEntries.group('hour')
                                queueid   = cleanupLineEntries.group('queueid')
                                messageid = cleanupLineEntries.group('messageid').replace("<","").replace(">","")
                                                                
                                data[queueid]['msgid'] = messageid
                                
                                if 'server' in data[queueid].keys():
                                        oldServer = data[queueid]['server']
                                        data[queueid]['server'] = [(date, hour, messageid, 'server', oldServer, '')]
                                
                                if surveyMessageid[messageid]:
                                        pass
                                else:
                                        surveyMessageid[messageid] = queueid
                                
                                if ( (self.searchCase == "messageid") and (surveyMessageid[messageid] == queueid) ):
                                        
                                        if self.searchPattern.match(messageid) is not None:
                                                surveyQueuid.append(queueid)
                                next
                        
                        
                        filenameLineEntries   = self.filenameLine.match(thisLine)
                        if filenameLineEntries is not None:
                                
                                date     = filenameLineEntries.group('date')
                                hour     = filenameLineEntries.group('hour')
                                queueid  = filenameLineEntries.group('queueid')
                                filename = filenameLineEntries.group('filename')
                                
                                if 'msgid' in data[queueid].keys():
                                        tupl = (date, hour, data[queueid]['msgid'], 'file', filename, '')
                                        try :
                                                data[queueid]['filename'].append(tupl)
                                        except KeyError: 
                                                data[queueid]['filename'] = []
                                                data[queueid]['filename'].append(tupl)
                                
                                
                                next
                                
                        qmgrLineEntries       = self.qmgrLine.match(thisLine)
                        if qmgrLineEntries is not None:
                                date     = qmgrLineEntries.group('date')
                                hour     = qmgrLineEntries.group('hour')
                                queueid  = qmgrLineEntries.group('queueid')
                                thisFrom = qmgrLineEntries.group('from')
                                size     = qmgrLineEntries.group('size')
                                nrcpt    = qmgrLineEntries.group('nrcpt')
                                 
                                data[queueid]['size']   =  [(date, hour, messageid, 'size', size, '')]
                                data[queueid]['nrcpt']  =  [(date, hour, messageid, 'nrcpt', nrcpt, '')]
                                data[queueid]['from'] =  [(date, hour, messageid, 'from', thisFrom, '')]
                                
                                # Because from address is the activator
                                if (self.searchCase == "from" or self.searchCase == "address"):
                                        if self.searchPattern.match(thisFrom) is not None:
                                                if( (queueid not in surveyQueuid) and (queueid in surveyMessageid.values()) ): 
                                                        surveyQueuid.append(queueid)
                                
                                next
                                
                                
                        smtpLineDSN2Entries   = self.smtpLineDSN2.match(thisLine)        
                        if smtpLineDSN2Entries is not None:
                                date      = smtpLineDSN2Entries.group('date')
                                hour      = smtpLineDSN2Entries.group('hour')
                                queueid   = smtpLineDSN2Entries.group('queueid')
                                recip     = smtpLineDSN2Entries.group('recip')
                                orig      = smtpLineDSN2Entries.group('orig')

                                # Because recip address is the activator 
                                if (self.searchCase == "recip" or self.searchCase == "address"):
                                        if self.searchPattern.match(recip) is not None:
                                                if( (queueid not in surveyQueuid) and (queueid in surveyMessageid.values()) ): 
                                                        surveyQueuid.append(queueid)
                                
                                        
                                # Any Case                
                                tupl = (date, hour, messageid, 'recip', recip, 'Relay OK')

                                
                                if queueid in surveyMessageid.values():
                                        try:
                                                data[queueid]['recip'].append(tupl)
                                        except KeyError: 
                                                data[queueid]['recip'] = []
                                                data[queueid]['recip'].append(tupl)
                                
                                next
                                
                                
                        smtpLineDSN4Entries   = self.smtpLineDSN4.match(thisLine)        
                        if smtpLineDSN4Entries is not None:
                                date     = smtpLineDSN4Entries.group('date')
                                hour     = smtpLineDSN4Entries.group('hour')
                                queueid  = smtpLineDSN4Entries.group('queueid')
                                recip    = smtpLineDSN4Entries.group('recip')
                                orig     = smtpLineDSN4Entries.group('orig')
                                
                                # Because recip address is the activator 
                                if (self.searchCase == "recip" or self.searchCase == "address"):
                                        if self.searchPattern.match(recip) is not None:
                                                if( (queueid not in surveyQueuid) and (queueid in surveyMessageid.values()) ):
                                                        surveyQueuid.append(queueid)
                                
                                # Any Case                
                                tupl = (date, hour, messageid, 'recip', recip, 'Deferred')
                                if queueid in surveyMessageid.values():
                                        try:
                                                data[queueid]['recip'].append(tupl)
                                        except KeyError: 
                                                data[queueid]['recip'] = []
                                                data[queueid]['recip'].append(tupl)
                                next
                                
                                
                        smtpLineDSN5Entries   = self.smtpLineDSN5.match(thisLine)        
                        if smtpLineDSN5Entries is not None:
                                date      = smtpLineDSN5Entries.group('date')
                                hour      = smtpLineDSN5Entries.group('hour')
                                queueid   = smtpLineDSN5Entries.group('queueid')
                                recip     = smtpLineDSN5Entries.group('recip')
                                orig      = smtpLineDSN5Entries.group('orig')

                                # Because recip address is the activator 
                                if (self.searchCase == "recip" or self.searchCase == "address"): 
                                        if self.searchPattern.match(recip) is not None:
                                                if( (queueid not in surveyQueuid) and (queueid in surveyMessageid.values()) ): 
                                                        surveyQueuid.append(queueid)
                                                
                                # Any Case                
                                tupl = (date, hour, messageid, 'recip', recip, 'Bounced')                
                                if queueid in surveyMessageid.values():
                                        try:
                                                data[queueid]['recip'].append(tupl)
                                        except KeyError: 
                                                data[queueid]['recip'] = []
                                                data[queueid]['recip'].append(tupl)
                                                        
                                next
                                
                                
                        smtpRemovedEntries = self.smtpRemoved.match(thisLine)
                        if smtpRemovedEntries is not None:
                                date      = smtpRemovedEntries.group('date')
                                hour      = smtpRemovedEntries.group('hour')
                                queueid   = smtpRemovedEntries.group('queueid')
                                                                                                
                                if queueid in surveyQueuid:
                                        for tkey in ('server', 'size','nrcpt','from'):
                                                
                                                if tkey in data[queueid]:
                                                        tupl = data[queueid][tkey]
                                                        self.cDb.insertIntoLog(tupl)
                                        
                                        if data[queueid]['recip'] is not None:
                                                self.cDb.insertIntoLog(data[queueid]['recip'])
                                        
                                        if 'filename' in data[queueid]:
                                                for tfilename in data[queueid]['filename']:
                                                        self.cDb.insertIntoLog([tfilename])
                                        
                                try:
                                        del data[queueid] 
                                except KeyError:
                                        pass
                                
                                next
                                
                                
                        cyrusDeliveredEntries = self.cyrusDelivered.match(thisLine)        
                        if cyrusDeliveredEntries is not None:
                                date      = cyrusDeliveredEntries.group('date')
                                hour      = cyrusDeliveredEntries.group('hour')
                                messageid = cyrusDeliveredEntries.group('messageid').replace("<","").replace(">","")
                                domain    = cyrusDeliveredEntries.group('domain')
                                user      = cyrusDeliveredEntries.group('user')
                                
                                recip = user.replace("^", ".") + "@" + domain 
                                
                                
                                if surveyMessageid[messageid] in surveyQueuid:
                                        tupl = [(date, hour, messageid, 'Delivery', recip, 'In Box')]
                                        self.cDb.insertIntoLog(tupl)
                                next
                                
                
        def getCountEntry(self):
                return self.cDb.getCountEntry()

        def closeSQL(self):
                self.cDb.close()
