#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
 * ------------------------------------------------------------------------
 * bmTraces
 * ------------------------------------------------------------------------
 * @license : GNU Affero General Public License Version 3 <http://www.gnu.org/licenses>
 * @author  : Pascal SALAUN 
 * @Website : http://www.bm-stats.org <http://www.bm-stats.org>
 * ------------------------------------------------------------------------
"""

# Native Python Lib/Class 
from Tkinter import * 
import ttk

# bmTraces Class
from sqlQuery import *


KB = 1024.0
MB = KB * KB
GB = MB * KB


class viewData:

        def __init__(self,userOwnDb, left, right):
                self.user      = userOwnDb
                self.cDb       = sqlQuery(self.user)
                self.left      = left
                self.right     = right
                
                
        def _format_size(self, size):
                if size >= GB:
                        return '{:,.1f} GB'.format(size/GB)
                if size >= MB:
                        return '{:,.1f} MB'.format(size/MB)
                if size >= KB:
                        return '{:,.1f} KB'.format(size/KB)
                return '{} bytes'.format(size)        
        
        
        def getList(self, cas):
                self.thisCase = thisCase = cas
                cDb = sqlQuery(self.user)
                self.messageList = cDb.getMessageList(cas)
                cDb.close()
        
        def treeviewSortColumn(self, tv, col, reverse):
                l = [(tv.set(k, col), k) for k in tv.get_children('')]
                l.sort(key=lambda t: t[0], reverse=reverse)
                
                for index, (val, k) in enumerate(l):
                        tv.move(k, '', index)
                        
                tv.heading(col,command=lambda: self.treeviewSortColumn(tv, col, not reverse))
        
        def displayList(self):
                left = self.left 
                
                scrollbar = Scrollbar(left)
                scrollbar.pack( side = RIGHT, fill=Y )
                
                titles = {'from':'Sender', 'recip':'Recipient', 'messageid':'Message-ID', 'address':'Address'}
                title  = titles[self.thisCase[0]]
                columns = ('Date','Hour', title )
                
                self.treeview = ttk.Treeview(self.left , height=50, columns=columns, show='headings')
                self.treeview['yscroll'] = scrollbar.set
                
                
                
                for tupl in self.messageList:
                        newTupl = (tupl[0],tupl[1],tupl[3])
                        messageid = tupl[2]
                        self.treeview.insert('', END, values=newTupl, text=messageid)
                self.treeview.pack()
                scrollbar.config( command = self.treeview.yview )
                
                self.treeview.heading('Date', text='Date', command=lambda: self.treeviewSortColumn(self.treeview, 'Date', False))
                self.treeview.column('Date',minwidth=0,width=75,)
                
                self.treeview.heading('Hour', text='Hour', command=lambda: self.treeviewSortColumn(self.treeview, 'Hour', False))
                self.treeview.column('Hour',minwidth=0,width=75,)
                
                self.treeview.heading(title, text=title, command=lambda: self.treeviewSortColumn(self.treeview, title, False))
                self.treeview.column(title,minwidth=0, width=300, anchor="w", )
                
                self.treeview.bind("<Double-1>", self.display)

                
        def display(self, event):
                """
                sql format : (id, dat, hour, messageid, cas, data, status)
                """
                item = self.treeview.identify('item',event.x,event.y)
                messageid = self.treeview.item(item,"text")
                tupl = (messageid,)
                
                cDb = sqlQuery(self.user)
                res = cDb.getMessageDetails(tupl)
                cDb.close()
                
                for slave in self.right.pack_slaves():
                        slave.destroy()
                        
                displayKey = {'server': 'Server (Connection by)','size':'Size','nrcpt':'Recipient Quantity' ,'from':'Sender', 'recip':'Recipient Transport Status', 'Delivery':'Delivery status OK (in box) for ', 'file': 'Attachments List' }
                Delivery   = [] 
                recip      = []
                files      = []
                
                for line in res:
                        
                        tkey = line[4]
                        if tkey == "server": 
                                # Date
                                T = LabelFrame(self.right, text="  Date :  ", relief='raise', bg="white", borderwidth=1 )
                                T.pack(fill=X, padx=15, pady=5)
                                                
                                valueText = "The " + line[1] + " at " + line[2]
                                t = Label(T, text=valueText, padx=10, bg="white", anchor=W, justify=LEFT)
                                t.pack(fill=X, padx=15, pady=5)
                                
                                # Server
                                T = LabelFrame(self.right, text="  " + displayKey[tkey] + ' :  ', relief='raise', bg="white", borderwidth=1 )
                                T.pack(fill=X, padx=15, pady=5)
                                
                                valueText = line[5]
                                t = Label(T, text=valueText, padx=10, bg="white", anchor=W, justify=LEFT)
                                t.pack(fill=X, padx=15, pady=5)
                                
                                # Message-ID
                                T = LabelFrame(self.right, text="  Message-ID :  ", relief='raise', bg="white", borderwidth=1 )
                                T.pack(fill=X, padx=15, pady=5)
                                        
                                valueText = line[3]
                                t = Label(T, text=valueText, padx=10, bg="white", anchor=W, justify=LEFT)
                                t.pack(fill=X, padx=15, pady=5)
                        
                        
                        
                        if tkey in ('size', 'nrcpt', 'from'):
                                T = LabelFrame(self.right, text="  " + displayKey[tkey] + ' :  ', relief='raise', bg="white", borderwidth=1 )
                                T.pack(fill=X, padx=15, pady=5)
                                
                                if tkey == 'size':
                                        valueText = self._format_size(int(line[5]))
                                else:
                                        valueText = line[5]
                                        
                                t = Label(T, text=valueText, padx=10, bg="white", anchor=W, justify=LEFT)
                                t.pack(fill=X, padx=15, pady=5)
                                
                        
                        
                        if tkey == "recip":
                                valueText = line[5] + " ("  + line[6] + ")"
                                recip.append(valueText)
                                
                        if tkey == "Delivery": 
                                Delivery.append(line[5])
                                
                        if tkey == "file":
                                files.append(line[5])
                
                
                if files:
                        fileText = ""
                        for line in files:
                                fileText += "-  " + line + "\n"
                        
                        T = LabelFrame(self.right, text="  " + displayKey["file"] + ' :  ', relief='raise', bg="white", borderwidth=1 )
                        T.pack(fill=X, padx=15, pady=5)
                                
                        t = Label(T, text=fileText, padx=10, bg="white", anchor=W, justify=LEFT)
                        t.pack(fill=X, padx=15, pady=5)                
                
                
                                
                if recip:
                        recipText = ""
                        for line in recip:
                                recipText += "-  " + line + "\n"
                        
                        T = LabelFrame(self.right, text="  " + displayKey["recip"] + ' :  ', relief='raise', bg="white", borderwidth=1 )
                        T.pack(fill=X, padx=15, pady=5)
                                
                        t = Label(T, text=recipText, padx=10, bg="white", anchor=W, justify=LEFT)
                        t.pack(fill=X, padx=15, pady=5)
                        
                if Delivery:
                        deliveryText = ""
                        for line in Delivery:
                                deliveryText += "-  " + line + "\n"
                        
                        T = LabelFrame(self.right, text="  " + displayKey["Delivery"] + ' :  ', relief='raise', bg="white", borderwidth=1 )
                        T.pack(fill=X, padx=15, pady=5)
                                
                        t = Label(T, text=deliveryText, padx=10, bg="white", anchor=W, justify=LEFT)
                        t.pack(fill=X, padx=15, pady=5)                        
                        
                        
